from balepy.objects import HTTPMethod

import balepy

class SendVoice:

    async def send_voice(
            self: "balepy.Client",
            chat_id: str,
            voice: str,
            caption=None,
            reply_to_message_id=None,
            reply_markup=None
    ):
        params = {
            'chat_id': chat_id,
            'voic': voice,
            'caption': caption,
            'reply_to_message_id': reply_to_message_id,
            'reply_markup': reply_markup
        }

        return await self.api.execute(name='sendVoice', method=HTTPMethod.POST, data=params)



from balepy.objects import HTTPMethod

import balepy


class EditMessageText:

    async def edit_message_text(
            self: "balepy.Client",
            text: str,
            chat_id: str = None,
            message_id: int = None,
            reply_markup=None
    ):
        params = {
            'chat_id': chat_id,
            'message_id': message_id,
            'text': text,
            'reply_markup': reply_markup
        }
        return await self.api.execute(name='editMessageText', method=HTTPMethod.POST, data=params)



from .send_message import SendMessage
from .forward_message import ForwardMessage
from .copy_message import CopyMessage
from .delete_message import DeleteMessage
from .edit_message_text import EditMessageText
from .send_photo import SendPhoto
from .send_audio import SendAudio
from .send_document import SendDocument
from .send_video import SendVideo
from .send_animation import SendAnimation
from .send_voice import SendVoice
from .send_media_group import SendMediaGroup
from .send_location import SendLocation


class Messages(SendMessage,
               ForwardMessage,
               CopyMessage,
               DeleteMessage,
               EditMessageText,
               SendPhoto,
               SendAudio,
               SendDocument,
               SendVideo,
               SendAnimation,
               SendVoice,
               SendMediaGroup,
               SendLocation
               ):
    pass



from balepy.objects import HTTPMethod

import balepy


class GetWebhookInfo:

    """
    for get status webhook
    """

    async def get_webhook_info(
            self: "balepy.Client",
            url: str
    ):
        params = {
            'url': url
        }
        return await self.api.execute(name="getWebhookInfo", method=HTTPMethod.POST, data=params)



from typing import Optional

from balepy.http import API
from balepy.types import Updates
from balepy.methods import Methods


class Client(Methods):

    def __init__(
            self,
            name: str,
            bot_token: str,
            timeout: Optional[int] = 20,
            max_retry: Optional[int] = 3,
            base_url: Optional[str] = None,
            proxies: Optional[str] = None
    ):
        """
        Initialize the Client instance.

        :param name: The name of the bot.
        :param bot_token: The bot token for authentication.
        :param base_url: The base URL for the Bale API.
        :param max_retry: The number of attempts by the client to send the request to the server.
        """
        self.name = name
        self.bot_token = bot_token
        self.timeout = timeout
        self.max_retry = max_retry
        self.base_url = base_url
        self.proxies = proxies
        self.api = API(client=self)

    def on_message(self, func):
        async def decorator():
            async for update in self.process_update():
                await func(update)
        return decorator

    async def process_update(self):
        offset = -1

        while True:
            update = await self.get_updates(offset=offset)

            offset = 1
            if not isinstance(update, str) and not len(update) == 0:
                break

        offset = update[-1]["update_id"] + 1
        while True:
            last_update = await self.get_updates(offset=offset)

            if not isinstance(last_update, str) and not len(last_update) == 0:
                offset += 1
                yield Updates(last_update[0])



from balepy.objects import HTTPMethod

import balepy


class SendMediaGroup:

    async def send_media_group(
            self: "balepy.Client",
            chat_id: str,
            media,
            reply_to_message_id=None
    ):
        params = {
            'chat_id': chat_id,
            'media': media,
            'reply_to_message_id': reply_to_message_id
        }
        return await self.api.execute(name="sendMediaGroup", method=HTTPMethod.POST, data=params)



from balepy.objects import HTTPMethod

import balepy


class Updates:

    def __init__(self, data: dict):
        self.data = data

    @property
    def message(self):
        if "callback_query" in self.data:
            return self.data["callback_query"]["message"]
        elif "message" in self.data:
            return self.data["message"]

    @property
    def is_callback(self):
        if "callback_query" in self.data:
            return True
        else:
            return False

    @property
    def update_id(self):
        return self.data["update_id"]

    @property
    def message_id(self):
        return self.message["message_id"]

    @property
    def author_id(self):
        return self.message["from"]["id"]

    @property
    def callback_author_id(self):
        if self.is_callback:
            return self.data["callback_query"]["from"]["id"]
        else:
            return None

    @property
    def is_bot(self):
        return self.message["from"]["is_bot"]

    @property
    def callback_is_bot(self):
        if self.is_callback:
            return self.data["callback_query"]["from"]["is_bot"]
        else:
            return None

    @property
    def author_first_name(self):
        return self.message["from"]["first_name"]

    @property
    def callback_author_first_name(self):
        if self.is_callback:
            return self.data["callback_query"]["from"]["first_name"]
        else:
            return None

    @property
    def author_last_name(self):
        return self.message["from"]["last_name"]

    @property
    def callback_author_last_name(self):
        if self.is_callback:
            return self.data["callback_query"]["from"]["last_name"]
        else:
            return None

    @property
    def author_username(self):
        if "username" in self.message["from"]:
            return self.message["from"]["username"]
        else:
            return None

    @property
    def callback_author_username(self):
        if self.is_callback:
            if "username" in self.data["callback_query"]:
                return self.data["callback_query"]["from"]["username"]
            else:
                return None
        else:
            return None

    @property
    def chat_id(self):
        return self.message["chat"]["id"]

    @property
    def chat_type(self):
        return self.message["chat"]["type"]

    @property
    def chat_title(self):
        return self.message["chat"]["title"]

    @property
    def new_member(self):
        if "new_chat_members" in self.message:
            return True
        else:
            return False

    @property
    def new_member_id(self):
        if self.new_member:
            return self.message["new_chat_members"][0]["id"]
        else:
            return None

    @property
    def new_member_is_bot(self):
        if self.new_member:
            return self.message["new_chat_members"][0]["is_bot"]
        else:
            return None

    @property
    def new_member_first_name(self):
        if self.new_member:
            return self.message["new_chat_members"][0]["first_name"]
        else:
            return None

    @property
    def new_member_last_name(self):
        if self.new_member:
            return self.message["new_chat_members"][0]["last_name"]
        else:
            return None

    @property
    def new_member_username(self):
        if self.new_member and "username" in self.message["new_chat_members"][0]:
            return self.message["new_chat_members"][0]["username"]
        else:
            return None

    @property
    def left_member(self):
        if "left_chat_member" in self.message:
            return True
        else:
            return False

    @property
    def left_member_id(self):
        if self.left_member:
            return self.message["left_chat_member"]["id"]
        else:
            return None

    @property
    def left_member_is_bot(self):
        if self.left_member:
            return self.message["left_chat_member"]["is_bot"]
        else:
            return None

    @property
    def left_member_first_name(self):
        if self.left_member:
            return self.message["left_chat_member"]["first_name"]
        else:
            return None

    @property
    def left_member_last_name(self):
        if self.left_member:
            return self.message["left_chat_member"]["last_name"]
        else:
            return None

    @property
    def left_member_username(self):
        if self.left_member and "username" in self.message["left_chat_member"]:
            return self.message["left_chat_member"]["username"]
        else:
            return None

    @property
    def is_reply(self):
        if "reply_to_message" in self.message:
            return True
        else:
            return False

    @property
    def reply_message_id(self):
        if "reply_to_message" in self.message:
            return self.message["reply_to_message"]["message_id"]
        else:
            return False

    @property
    def reply_data(self):
        if "reply_to_message" in self.message:
            return self.message["reply_to_message"]
        else:
            return False

    @property
    def is_forward(self):
        if "forward_from" in self.message:
            return True
        elif "forward_from_chat" in self.message:
            return True
        else:
            return False

    @property
    def forward_from_id(self):
        if "forward_from" in self.message:
            return self.message["forward_from"]["id"]
        elif "forward_from_chat" in self.message:
            return self.message["forward_from_chat"]["id"]
        else:
            return None

    @property
    def forward_from_message_id(self):
        if "forward_from_message_id" in self.message:
            return self.message["forward_from_message_id"]
        else:
            return None

    @property
    def forward_from_is_bot(self):
        if "forward_from" in self.message:
            return self.message["forward_from"]["is_bot"]
        else:
            return None

    @property
    def forward_from_is_bot(self):
        if self.is_forward:
            return self.message["forward_from"]["is_bot"]
        else:
            return None

    @property
    def text(self):
        if "text" in self.message:
            return self.message["text"]
        elif "caption" in self.message:
            return self.message["caption"]
        else:
            return None

    @property
    def callback_data_send(self):
        if self.is_callback:
            return self.data["callback_query"]["data"]
        else:
            return None

    @property
    def file_data(self):
        if "photo" in self.message:
            return self.message["photo"]
        elif "video" in self.message:
            return self.message["video"]
        elif "document" in self.message:
            return self.message["document"]
        elif "audio" in self.message:
            return self.message["audio"]
        else:
            return None

    @property
    def message_type(self):
        if "text" in self.message:
            return "text"
        elif "photo" in self.message:
            if "caption" in self.message:
                return "photoCaption"
            else:
                return "photo"
        elif "video" in self.message:
            if "caption" in self.message:
                return "videoCaption"
            else:
                return "video"
        elif "document" in self.message:
            if "caption" in self.message:
                return "documentCaption"
            else:
                return "document"
        elif "audio" in self.message:
            if "caption" in self.message:
                return "audioCaption"
            else:
                return "audio"
        else:
            return None

    async def reply(self: "balepy.Client", text: str) -> dict:
        payload: dict = {
            "text": text,
            "chat_id": Updates.chat_id,
            "reply_to_message_id": Updates.message_id
        }
        return await self.api.execute(name="sendMessage", method=HTTPMethod.POST, data=payload)

    @property
    def is_groups(self):
        if self.message["chat"]["type"] == "group":
            return self.message

    @property
    def is_channel(self):
        if self.message["chat"]["type"] == "channel":
            return self.message

    @property
    def is_private(self):
        if self.message["chat"]["type"] == "private":
            return self.message




class APIError(Exception):
    """
    Signal error with an API access.
    """
    error_code: int
    description: str

    def __post_init__(self):
        super().__init__(f"Error {self.error_code}: {self.description}")



from balepy.objects import HTTPMethod

import balepy

from typing import Optional


class GetUpdates:

    async def get_updates(
            self: "balepy.Client",
            offset: Optional[int] = -1,
            limit: Optional[int] = 1
    ):
        params: dict = {
            "offset": offset,
            "limit": limit
        }
        response_data = await self.api.execute(name="getUpdates", method=HTTPMethod.POST, data=params)
        return response_data["result"]



from balepy.objects import HTTPMethod

import balepy


class SendMessage:

    async def send_message(
            self: "balepy.Client",
            chat_id: str,
            text: str,
            reply_to_message_id: int = None,
            reply_markup=None
    ):
        params = {
            "chat_id": chat_id,
            "text": text,
            "reply_to_message_id": reply_to_message_id,
            "reply_markup": reply_markup
        }
        return await self.api.execute(name="sendMessage", method=HTTPMethod.POST, data=params)



from balepy.objects import HTTPMethod

import balepy


class SendPhoto:

    async def send_photo(
            self: "balepy.Client",
            chat_id: str,
            from_chat_id: str,
            photo: str,
            caption=None,
            reply_to_message_id=None,
            reply_markup=None
    ):
        params = {
            'chat_id': chat_id,
            'from_chat_id': from_chat_id,
            'photo': photo,
            'caption': caption,
            'reply_to_message_id': reply_to_message_id,
            'reply_markup': reply_markup
        }
        return await self.api.execute(name="sendPhoto", method=HTTPMethod.POST, data=params)



from balepy.objects import HTTPMethod

import balepy


class CopyMessage:

    async def copy_message(
            self: "balepy.Client",
            chat_id: str,
            from_chat_id: str,
            message_id: int
    ):
        params = {
            'chat_id': chat_id,
            'from_chat_id': from_chat_id,
            'message_id': message_id
        }
        return await self.api.execute(name="copyMessage", method=HTTPMethod.POST, data=params)



from balepy.objects import HTTPMethod

import balepy


class SendAnimation:

    async def send_animation(
            self: "balepy.Client",
            chat_id: str,
            animation: str,
            reply_to_message_id=None,
            reply_markup=None
    ):
        params = {
            'chat_id': chat_id,
            'animation': animation,
            'reply_to_message_id': reply_to_message_id,
            'reply_markup': reply_markup
        }

        return await self.api.execute(name='sendAnimation', method=HTTPMethod.POST, data=params)



from setuptools import setup, find_packages

setup(
    name = 'balepy',
    version = '1.4.2',
    author='Mohammad, AmirAli',
    author_email = 'mohammadmehrabi175@gmail.com',
    description = 'balepy a Library Python for create bots API in bale application',
    keywords = ['bot' , 'Bot' , 'bale' , 'robot', 'messangers'],
    long_description = open("README.md", encoding="utf-8").read(),
    python_requires="~=3.6",
    long_description_content_type = 'text/markdown',
    url = 'https://github.com/MamdMehrabi/balepy',
    packages = find_packages(),
    install_requires = ['aiohttp'],
    classifiers = [
    	'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'License :: OSI Approved :: MIT License',
    ]
)



from balepy.objects import HTTPMethod

import balepy


class SetWebhook:

    '''
    this method for get updates on webhook
    '''

    async def set_webhook(
            self: "balepy.Client",
            url: str
    ):
        params = {
            'url': url
        }
        return await self.api.execute(name="setWebhook", method=HTTPMethod.POST, data=params)



from balepy.objects import HTTPMethod

import balepy


class SendLocation:

    async def send_location(
            self: "balepy.Client",
            chat_id: str,
            latitude: float,
            longitude: float,
            horizontal_accuracy=None,
            reply_to_message_id=None,
            reply_markup=None
    ):
        params = {
            'chat_id': chat_id,
            'latitude': latitude,
            'longitude': longitude,
            'horizontal_accuracy': horizontal_accuracy,
            'reply_to_message_id': reply_to_message_id,
            'reply_markup': reply_markup
        }

        await self.api.execute(name="sendLocation", method=HTTPMethod.POST, data=params)



from typing import Optional

import aiohttp

from balepy.types import Results
from balepy.errors import APIError


class API:

    BASE_URL = "https://tapi.bale.ai"

    def __init__(self, client=None):
        self.client = client

    async def execute(self, name: str, method: Optional[str] = "GET", data: Optional[dict] = None):
        """
        Execute asynchronous request to BaleAPI
        """
        base_url = self.client.base_url or self.BASE_URL
        path = f"/bot{self.client.bot_token}/" + name
        timeout = aiohttp.ClientTimeout(total=self.client.timeout)
        for _ in range(self.client.max_retry):
            async with aiohttp.ClientSession(base_url=base_url, timeout=timeout) as self.session:
                async with self.session.request(method=method, url=path, json=data) as response:
                    response_data = await response.json()
                    if response_data.get("ok"):
                        response_data.pop("ok")
                        return Results(response_data)
                    error_code = response_data.get("error_code")
                    description = response_data.get("description")
                    raise APIError(description, error_code)
